/**********************************************************************
*
* Copyright (c) 2012-2020 Barbara Geller
* Copyright (c) 2012-2020 Ansel Sermersheim
*
* You may use this file under the terms of the 2-Clause BSD license
* provided with PepperMill or available at:
*
* https://opensource.org/licenses/BSD-2-Clause
*
* PepperMill is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
*
***********************************************************************/

#ifndef PEPPERMILL_H
#define PEPPERMILL_H

#include <QByteArray>
#include <QMap>
#include <QSet>
#include <QString>
#include <QStringList>

#include <utility>

class PepperMill
{
   public:
      enum levels{ONEFILE, ONEFOLDER, RECURSIVE};

      PepperMill(enum levels id, QString path, bool forceBU);
      ~PepperMill();

      void process();

   private:
      void pmConvert(const QString &name);
      QByteArray pmParse(const QByteArray &input);
      QString pmToken(const QString &word, const char **ptr);

      QString pmGetData(const char **ptr);
      QString pmGetMethod(const char **ptr);
      std::tuple<QString, QList<QString>, QString> pmGetSignature(const QString &fullName);
      bool pmIs_WhiteSpace(char s);
      bool pmIs_LineBreak(char s);
      bool pmisToken_Method(const char **ptr);
      bool pmisToken_Slot(const char **ptr);
      QString pmPad(const char *ptr);

      bool m_processFriend;
      bool m_processClass;
      bool m_consume;

      QSet<QString> m_defines;
      QList<QString> m_regEnum;

      QString m_processWord;
      QString m_className;

      //
      QByteArray process(const QByteArray &input);
      void findRecursive(const QString &path);

      enum levels m_type;
      QString m_path;
      bool m_forceBU;

      QStringList m_fileList;

      // flags
      QMap<QString, QString> m_flagMap;

      // sections
      QSet<QString> m_sectionSet;

      enum class Section {
         PUBLIC,
         PROTECTED,
         PRIVATE,
         SIGNAL_PUBLIC,
         SLOT_PUBLIC,
         SLOT_PROTECTED,
         SLOT_PRIVATE,
         NONE
      };

      Section m_section;
};

void csMsg(const QString &msg);

#endif
